import React from 'react';

class HostIdenticator extends React.Component {
  render() {
    var label, className;
    if (this.props.https === null) {
      className = "unknown";
      label = "?";
    } else if (this.props.https) {
      className = "https";
      label = "https";
    } else {
      className = "http";
      label = "http";
    }

    return <div className={className}>{ label }</div>;
  }
}

class IndicatorPanel extends React.Component {
  render() {
    return (
      <div className="indicator-panel">
        {this.props.bits.map((bit, i) => {
          return (
              <HostIdenticator key={`host-ind-${i}`} https={ bit } />
          );
        })}
      </div>
      );
  }
}

class ContentIndicator extends React.Component {
  render() {
    var className = this.props.ready ? "ready" : "pending";
    var label = this.props.ready ? "Ready ✔" : "loading...";
    return (
      <div className="content-indicator">
        <div className={className}>{this.props.text}</div>
        <div className={className}>{label}</div>
      </div>
      );
  }

}

function pad(n, width, z) {
    z = z || '0';
      n = n + '';
        return n.length >= width ? n : new Array(width - n.length + 1).join(z) +
        n;
}

var format =  n => pad(n, 3);

class ComplexEncoder extends React.Component {
  constructor(props) {
    super(props);
    this.state = {bits: new Array(this.props.maxBit).fill(null), inputValue: ""};
    this.encodeInput = this.encodeInput.bind(this);
    this.updateInputValue = this.updateInputValue.bind(this);
    this.handleKeyPress = this.handleKeyPress.bind(this);
  }

  is_ready() {
    return this.state.bits.every((b) => (b !== null));
  }

  has_content() {
    return this.state.bits.some((b) => (b == true && b !== null));
  }

  build_callback(index) {
    return (res => {
        var d = this.state.bits;
        if (res.trim() == "http") {
          d[index] = false;
        } else if (res.trim() == "https") {
          d[index] = true;
          this.setState({all_false: false});
        } else {
          d[index] = null;
        }
        this.setState({bits: d});
        for (var i=0; i<this.props.maxBit; i++) {
          if (d[i] === null) {
            return;
          }
        }
        this.setState({ready: true});
    });
  }

  componentDidMount() {
    for (var i=0; i<this.props.maxBit; i++) {
      var key = "b" + format(i);
      fetch("http://" + key + ".tls-tracking.sauerburger.com")
        .then(res => res.text())
        .then(this.build_callback(i));
    }
  }

  encode(string) {
    var binary = "";
    for (var i=0; i<string.length; i++) {
      binary += pad(string.charCodeAt(i).toString(2), 8);
    }

    for (var i=0; i<binary.length && i < this.props.maxBit; i++) {
      if (binary[i] == "1") {
        var key = "b" + format(i);
        fetch("https://" + key + ".tls-tracking.sauerburger.com")
          .then(res => res.text())
          .then(this.build_callback(i));
      }
    }

  }

  decode() {
    var decoded = "";
    var binary = this.state.bits.map(x => {return x ? "1" : "0"}).join("");
    for (var i = 0; i < binary.length / 8; i++) {
      var charCode = parseInt(binary.substring(i * 8, (i+1) * 8), 2);
      if (charCode == 0) {
        decoded += "";
      } else {
        decoded += String.fromCharCode(charCode);
      }
    }
    return decoded;
  }

  encodeInput() {
    this.encode(this.state.inputValue);
  }

  updateInputValue(evt) {
    this.setState({inputValue: evt.target.value});
  }

  handleKeyPress(evt) {
    if (evt.key === "Enter") {
      this.encodeInput();
    }
  }

  render() {
        return (<div id="complexEncoder">
            <IndicatorPanel bits={this.state.bits} />
            { this.has_content() ? 
              (<div>Your browser is tracked with the following string:
                <ContentIndicator text={this.decode()} ready={this.is_ready()} />
               </div>) : 
               (<div><p>Your browser is not yet tracked. Enter an arbitary phrase
               to be encoded. </p>
                <p><input type="text" maxLength={this.props.maxBit / 8}
                onChange={this.updateInputValue}
                onKeyPress={this.handleKeyPress}/>
                <input type="button" value="Submit" onClick={this.encodeInput} />
                </p></div>)
            }
             <p> Please note, to remove the tracking tag, you need
             to clear the recent history in your browser.</p>
            </div>);

  }

}

export default class App extends React.Component {
  render() {
    return (
      <div id="content">
        <ComplexEncoder maxBit={128}/>
      </div>
    );
  }
}
